@@ -25,75 +25,77 @@
 #define __GST_H__
 
 #include <glib.h>
-#include <popt.h>
+
+#include <gst/glib-compat.h>
 
 #include <gst/gstenumtypes.h>
-#include <gst/gsttypes.h>
 #include <gst/gstversion.h>
 
 #include <gst/gstbin.h>
 #include <gst/gstbuffer.h>
 #include <gst/gstcaps.h>
+#include <gst/gstchildproxy.h>
 #include <gst/gstclock.h>
-#include <gst/gstcpu.h>
+#include <gst/gstdebugutils.h>
 #include <gst/gstelement.h>
 #include <gst/gsterror.h>
 #include <gst/gstevent.h>
+#include <gst/gstghostpad.h>
 #include <gst/gstindex.h>
+#include <gst/gstindexfactory.h>
 #include <gst/gstinfo.h>
 #include <gst/gstinterface.h>
+#include <gst/gstiterator.h>
 #include <gst/gstmarshal.h>
+#include <gst/gstmessage.h>
+#include <gst/gstminiobject.h>
 #include <gst/gstobject.h>
 #include <gst/gstpad.h>
+#include <gst/gstparamspecs.h>
 #include <gst/gstpipeline.h>
 #include <gst/gstplugin.h>
-#include <gst/gstscheduler.h>
+#include <gst/gstpoll.h>
+#include <gst/gstquery.h>
+#include <gst/gstregistry.h>
+#include <gst/gstsegment.h>
 #include <gst/gststructure.h>
 #include <gst/gstsystemclock.h>
-#include <gst/gsttag.h>
-#include <gst/gsttaginterface.h>
-#include <gst/gstthread.h>
+#include <gst/gsttaglist.h>
+#include <gst/gsttagsetter.h>
+#include <gst/gsttask.h>
 #include <gst/gsttrace.h>
 #include <gst/gsttypefind.h>
+#include <gst/gsttypefindfactory.h>
 #include <gst/gsturi.h>
-#include <gst/gsturitype.h>
 #include <gst/gstutils.h>
 #include <gst/gstvalue.h>
 #include <gst/gstxml.h>
 
 #include <gst/gstparse.h>
-#include <gst/gstregistry.h>
-#include <gst/gstregistrypool.h>
 
 /* API compatibility stuff */
 #include <gst/gstcompat.h>
 
 G_BEGIN_DECLS
 
-/* make our own type for poptOption because the struct poptOption
- * definition is iffy */
-typedef struct poptOption GstPoptOption;
-
-/* initialize GST */
 void		gst_init			(int *argc, char **argv[]);
-gboolean	gst_init_check			(int *argc, char **argv[]);
-void		gst_init_with_popt_table	(int *argc, char **argv[],
-						 const GstPoptOption
-						 *popt_options);
-gboolean	gst_init_check_with_popt_table	(int *argc, char **argv[],
-						 const GstPoptOption
-						 *popt_options);
+gboolean	gst_init_check			(int *argc, char **argv[],
+						 GError ** err);
+GOptionGroup *	gst_init_get_option_group	(void);
+void		gst_deinit			(void);
 
-const GstPoptOption *	gst_init_get_popt_table		(void);
+void		gst_version			(guint *major, guint *minor,
+						 guint *micro, guint *nano);
+gchar *		gst_version_string		(void);
 
-void		gst_use_threads			(gboolean use_threads);
-gboolean	gst_has_threads			(void);
+gboolean        gst_segtrap_is_enabled          (void);
+void            gst_segtrap_set_enabled         (gboolean enabled);
 
-void		gst_main			(void);
-void		gst_main_quit			(void);
+gboolean        gst_registry_fork_is_enabled    (void);
+void            gst_registry_fork_set_enabled   (gboolean enabled);
+
+gboolean        gst_update_registry (void);
 
 G_END_DECLS
 
-#include <gst/gstlog.h>
-
 #endif /* __GST_H__ */
