@@ -39,10 +39,11 @@ extern const char             g_log_domain_gstreamer[];
 
 /* Needed for GstRegistry * */
 #include "gstregistry.h"
+#include "gststructure.h"
 
 G_BEGIN_DECLS
 
-gboolean __gst_in_valgrind (void);
+gboolean _priv_gst_in_valgrind (void);
 
 /* Initialize GStreamer private quark storage */
 void _priv_gst_quarks_initialize (void);
@@ -65,6 +66,24 @@ void  _gst_value_initialize (void);
 gboolean _priv_gst_registry_remove_cache_plugins (GstRegistry *registry);
 void _priv_gst_registry_cleanup (void);
 
+/* used in both gststructure.c and gstcaps.c; numbers are completely made up */
+#define STRUCTURE_ESTIMATED_STRING_LEN(s) (16 + (s)->fields->len * 22)
+
+gboolean  priv_gst_structure_append_to_gstring (const GstStructure * structure,
+                                                GString            * s);
+
+/* registry cache backends */
+/* FIXME 0.11: use priv_ prefix */
+#ifdef USE_BINARY_REGISTRY
+gboolean 		gst_registry_binary_read_cache 	(GstRegistry * registry, const char *location);
+gboolean 		gst_registry_binary_write_cache	(GstRegistry * registry, const char *location);
+/* FIXME 0.11: this is in registry.h for backwards compatibility
+#else 
+gboolean 		gst_registry_xml_read_cache 	(GstRegistry * registry, const char *location);
+gboolean 		gst_registry_xml_write_cache 	(GstRegistry * registry, const char *location);
+*/
+#endif
+
 /*** debugging categories *****************************************************/
 
 #ifndef GST_DISABLE_GST_DEBUG
@@ -109,6 +128,7 @@ extern IMPORT_SYMBOL GstDebugCategory *GST_CAT_SIGNAL;
 extern IMPORT_SYMBOL GstDebugCategory *GST_CAT_PROBE;
 extern IMPORT_SYMBOL GstDebugCategory *GST_CAT_REGISTRY;
 extern IMPORT_SYMBOL GstDebugCategory *GST_CAT_QOS;
+extern IMPORT_SYMBOL GstDebugCategory *GST_CAT_TYPES; /* FIXME 0.11: remove? */
 
 #else
 
@@ -141,6 +161,7 @@ extern IMPORT_SYMBOL GstDebugCategory *GST_CAT_QOS;
 #define GST_CAT_PROBE            NULL
 #define GST_CAT_REGISTRY         NULL
 #define GST_CAT_QOS              NULL
+#define GST_CAT_TYPES            NULL
 
 #endif
 
