@@ -20,13 +20,25 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:gstindex
+ * @short_description: Generate indexes on objects
+ * @see_also: #GstIndexFactory
+ *
+ * GstIndex is used to generate a stream index of one or more elements
+ * in a pipeline.
+ */
+
 #include "gst_private.h"
 
 #include "gstinfo.h"
-#include "gstregistrypool.h"
-#include "gstpad.h"
 #include "gstindex.h"
+#include "gstindexfactory.h"
 #include "gstmarshal.h"
+#include "gstregistry.h"
+/* for constructing an entry name */
+#include "gstelement.h"
+#include "gstpad.h"
 
 /* Index signals and args */
 enum
@@ -44,6 +56,7 @@ enum
 
 static void gst_index_class_init (GstIndexClass * klass);
 static void gst_index_init (GstIndex * index);
+static void gst_index_finalize (GObject * object);
 
 static void gst_index_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -80,7 +93,7 @@ static GType
 gst_index_resolver_get_type (void)
 {
   static GType index_resolver_type = 0;
-  static GEnumValue index_resolver[] = {
+  static const GEnumValue index_resolver[] = {
     {GST_INDEX_RESOLVER_CUSTOM, "GST_INDEX_RESOLVER_CUSTOM",
         "Use a custom resolver"},
     {GST_INDEX_RESOLVER_GTYPE, "GST_INDEX_RESOLVER_GTYPE",
@@ -140,13 +153,18 @@ static void
 gst_index_class_init (GstIndexClass * klass)
 {
   GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
 
-  gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
+  gobject_class = G_OBJECT_CLASS (klass);
 
-  parent_class = g_type_class_ref (GST_TYPE_OBJECT);
+  parent_class = g_type_class_peek_parent (klass);
 
+  /**
+   * GstIndex::entry-added
+   * @gstindex: the object which received the signal.
+   * @arg1: The entry added to the index.
+   *
+   * Is emitted when a new entry is added to the index.
+   */
   gst_index_signals[ENTRY_ADDED] =
       g_signal_new ("entry-added", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstIndexClass, entry_added), NULL, NULL,
@@ -154,8 +172,9 @@ gst_index_class_init (GstIndexClass * klass)
 
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_index_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_index_get_property);
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_index_finalize);
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_RESOLVER,
+  g_object_class_install_property (gobject_class, ARG_RESOLVER,
       g_param_spec_enum ("resolver", "Resolver",
           "Select a predefined object to string mapper",
           GST_TYPE_INDEX_RESOLVER, GST_INDEX_RESOLVER_PATH, G_PARAM_READWRITE));
@@ -175,12 +194,48 @@ gst_index_init (GstIndex * index)
   index->resolver = resolvers[index->method].resolver;
   index->resolver_user_data = resolvers[index->method].user_data;
 
-  GST_FLAG_SET (index, GST_INDEX_WRITABLE);
-  GST_FLAG_SET (index, GST_INDEX_READABLE);
+  GST_OBJECT_FLAG_SET (index, GST_INDEX_WRITABLE);
+  GST_OBJECT_FLAG_SET (index, GST_INDEX_READABLE);
 
   GST_DEBUG ("created new index");
 }
 
+static void
+gst_index_free_writer (gpointer key, gpointer value, gpointer user_data)
+{
+  GstIndexEntry *entry = (GstIndexEntry *) value;
+
+  if (entry) {
+    gst_index_entry_free (entry);
+  }
+}
+
+static void
+gst_index_finalize (GObject * object)
+{
+  GstIndex *index = GST_INDEX (object);
+
+  if (index->groups) {
+    g_list_foreach (index->groups, (GFunc) g_free, NULL);
+    g_list_free (index->groups);
+    index->groups = NULL;
+  }
+
+  if (index->writers) {
+    g_hash_table_foreach (index->writers, gst_index_free_writer, NULL);
+    g_hash_table_destroy (index->writers);
+    index->writers = NULL;
+  }
+
+  if (index->filter_user_data && index->filter_user_data_destroy)
+    index->filter_user_data_destroy (index->filter_user_data);
+
+  if (index->resolver_user_data && index->resolver_user_data_destroy)
+    index->resolver_user_data_destroy (index->resolver_user_data);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static void
 gst_index_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -383,8 +438,31 @@ gst_index_set_filter (GstIndex * index,
 {
   g_return_if_fail (GST_IS_INDEX (index));
 
+  gst_index_set_filter_full (index, filter, user_data, NULL);
+}
+
+/**
+ * gst_index_set_filter_full:
+ * @index: the index to register the filter on
+ * @filter: the filter to register
+ * @user_data: data passed to the filter function
+ * @user_data_destroy: function to call when @user_data is unset
+ *
+ * Lets the app register a custom filter function so that
+ * it can select what entries should be stored in the index.
+ */
+void
+gst_index_set_filter_full (GstIndex * index,
+    GstIndexFilter filter, gpointer user_data, GDestroyNotify user_data_destroy)
+{
+  g_return_if_fail (GST_IS_INDEX (index));
+
+  if (index->filter_user_data && index->filter_user_data_destroy)
+    index->filter_user_data_destroy (index->filter_user_data);
+
   index->filter = filter;
   index->filter_user_data = user_data;
+  index->filter_user_data_destroy = user_data_destroy;
 }
 
 /**
@@ -399,11 +477,34 @@ gst_index_set_filter (GstIndex * index,
 void
 gst_index_set_resolver (GstIndex * index,
     GstIndexResolver resolver, gpointer user_data)
+{
+  gst_index_set_resolver_full (index, resolver, user_data, NULL);
+}
+
+/**
+ * gst_index_set_resolver_full:
+ * @index: the index to register the resolver on
+ * @resolver: the resolver to register
+ * @user_data: data passed to the resolver function
+ * @user_data_destroy: destroy function for @user_data
+ *
+ * Lets the app register a custom function to map index
+ * ids to writer descriptions.
+ *
+ * Since: 0.10.18
+ */
+void
+gst_index_set_resolver_full (GstIndex * index, GstIndexResolver resolver,
+    gpointer user_data, GDestroyNotify user_data_destroy)
 {
   g_return_if_fail (GST_IS_INDEX (index));
 
+  if (index->resolver_user_data && index->resolver_user_data_destroy)
+    index->resolver_user_data_destroy (index->resolver_user_data);
+
   index->resolver = resolver;
   index->resolver_user_data = user_data;
+  index->resolver_user_data_destroy = user_data_destroy;
   index->method = GST_INDEX_RESOLVER_CUSTOM;
 }
 
@@ -430,6 +531,25 @@ gst_index_entry_copy (GstIndexEntry * entry)
 void
 gst_index_entry_free (GstIndexEntry * entry)
 {
+  switch (entry->type) {
+    case GST_INDEX_ENTRY_ID:
+      if (entry->data.id.description) {
+        g_free (entry->data.id.description);
+        entry->data.id.description = NULL;
+      }
+      break;
+    case GST_INDEX_ENTRY_ASSOCIATION:
+      if (entry->data.assoc.assocs) {
+        g_free (entry->data.assoc.assocs);
+        entry->data.assoc.assocs = NULL;
+      }
+      break;
+    case GST_INDEX_ENTRY_OBJECT:
+      break;
+    case GST_INDEX_ENTRY_FORMAT:
+      break;
+  }
+
   g_free (entry);
 }
 
@@ -515,10 +635,17 @@ gst_index_gtype_resolver (GstIndex * index, GstObject * writer,
     gchar ** writer_string, gpointer data)
 {
   if (GST_IS_PAD (writer)) {
-    GstElement *element = gst_pad_get_parent (GST_PAD (writer));
+    GstElement *element =
+        (GstElement *) gst_object_get_parent (GST_OBJECT (writer));
+    gchar *name;
 
+    name = gst_object_get_name (writer);
     *writer_string = g_strdup_printf ("%s.%s",
-        g_type_name (G_OBJECT_TYPE (element)), gst_object_get_name (writer));
+        g_type_name (G_OBJECT_TYPE (element)), name);
+
+    gst_object_unref (element);
+    g_free (name);
+
   } else {
     *writer_string =
         g_strdup_printf ("%s", g_type_name (G_OBJECT_TYPE (writer)));
@@ -535,9 +662,9 @@ gst_index_gtype_resolver (GstIndex * index, GstObject * writer,
  *
  * Before entries can be added to the index, a writer
  * should obtain a unique id. The methods to add new entries
- * to the index require this id as an argument. 
+ * to the index require this id as an argument.
  *
- * The application can implement a custom function to map the writer object 
+ * The application can implement a custom function to map the writer object
  * to a string. That string will be used to register or look up an id
  * in the index.
  *
@@ -623,7 +750,7 @@ gst_index_add_entry (GstIndex * index, GstIndexEntry * entry)
  * @id: the id of the index writer
  * @flags: optinal flags for this entry
  * @n: number of associations
- * @list: list of format/value associations pais
+ * @list: list of associations
  *
  * Associate given format/value pairs with each other.
  *
@@ -631,7 +758,7 @@ gst_index_add_entry (GstIndex * index, GstIndexEntry * entry)
  */
 GstIndexEntry *
 gst_index_add_associationv (GstIndex * index, gint id, GstAssocFlags flags,
-    int n, const GstIndexAssociation * list)
+    gint n, const GstIndexAssociation * list)
 {
   GstIndexEntry *entry;
 
@@ -661,7 +788,7 @@ gst_index_add_associationv (GstIndex * index, gint id, GstAssocFlags flags,
  * @id: the id of the index writer
  * @flags: optinal flags for this entry
  * @format: the format of the value
- * @value: the value 
+ * @value: the value
  * @...: other format/value pairs or 0 to end the list
  *
  * Associate given format/value pairs with each other.
@@ -689,22 +816,26 @@ gst_index_add_association (GstIndex * index, gint id, GstAssocFlags flags,
 
   array = g_array_new (FALSE, FALSE, sizeof (GstIndexAssociation));
 
-  va_start (args, value);
-
-  cur_format = format;
-  n_assocs = 0;
-  while (cur_format) {
+  {
     GstIndexAssociation a;
 
-    n_assocs++;
-    cur_format = va_arg (args, GstFormat);
-    if (cur_format) {
-      a.format = cur_format;
-      a.value = va_arg (args, gint64);
-
-      g_array_append_val (array, a);
-    }
+    a.format = format;
+    a.value = value;
+    n_assocs = 1;
+    g_array_append_val (array, a);
   }
+
+  va_start (args, value);
+
+  while ((cur_format = va_arg (args, GstFormat))) {
+    GstIndexAssociation a;
+
+    a.format = cur_format;
+    a.value = va_arg (args, gint64);
+    n_assocs++;
+    g_array_append_val (array, a);
+  }
+
   va_end (args);
 
   list = (GstIndexAssociation *) g_array_free (array, FALSE);
@@ -724,7 +855,9 @@ gst_index_add_association (GstIndex * index, gint id, GstAssocFlags flags,
  * @object: a pointer to the object to add
  *
  * Add the given object to the index with the given key.
- * 
+ *
+ * This function is not yet implemented.
+ *
  * Returns: a pointer to the newly added entry in the index.
  */
 GstIndexEntry *
@@ -821,7 +954,7 @@ gst_index_get_assoc_entry_full (GstIndex * index, gint id,
  *
  * Gets alternative formats associated with the indexentry.
  *
- * Returns: TRUE if there was a value associated with the given 
+ * Returns: TRUE if there was a value associated with the given
  * format.
  */
 gboolean
@@ -841,173 +974,3 @@ gst_index_entry_assoc_map (GstIndexEntry * entry,
   }
   return FALSE;
 }
-
-
-static void gst_index_factory_class_init (GstIndexFactoryClass * klass);
-static void gst_index_factory_init (GstIndexFactory * factory);
-
-static GstPluginFeatureClass *factory_parent_class = NULL;
-
-/* static guint gst_index_factory_signals[LAST_SIGNAL] = { 0 }; */
-
-GType
-gst_index_factory_get_type (void)
-{
-  static GType indexfactory_type = 0;
-
-  if (!indexfactory_type) {
-    static const GTypeInfo indexfactory_info = {
-      sizeof (GstIndexFactoryClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) gst_index_factory_class_init,
-      NULL,
-      NULL,
-      sizeof (GstIndexFactory),
-      0,
-      (GInstanceInitFunc) gst_index_factory_init,
-      NULL
-    };
-
-    indexfactory_type = g_type_register_static (GST_TYPE_PLUGIN_FEATURE,
-        "GstIndexFactory", &indexfactory_info, 0);
-  }
-  return indexfactory_type;
-}
-
-static void
-gst_index_factory_class_init (GstIndexFactoryClass * klass)
-{
-  GObjectClass *gobject_class;
-  GstObjectClass *gstobject_class;
-  GstPluginFeatureClass *gstpluginfeature_class;
-
-  gobject_class = (GObjectClass *) klass;
-  gstobject_class = (GstObjectClass *) klass;
-  gstpluginfeature_class = (GstPluginFeatureClass *) klass;
-
-  factory_parent_class = g_type_class_ref (GST_TYPE_PLUGIN_FEATURE);
-}
-
-static void
-gst_index_factory_init (GstIndexFactory * factory)
-{
-}
-
-/**
- * gst_index_factory_new:
- * @name: name of indexfactory to create
- * @longdesc: long description of indexfactory to create
- * @type: the GType of the GstIndex element of this factory
- *
- * Create a new indexfactory with the given parameters
- *
- * Returns: a new #GstIndexFactory.
- */
-GstIndexFactory *
-gst_index_factory_new (const gchar * name, const gchar * longdesc, GType type)
-{
-  GstIndexFactory *factory;
-
-  g_return_val_if_fail (name != NULL, NULL);
-  factory = gst_index_factory_find (name);
-  if (!factory) {
-    factory = GST_INDEX_FACTORY (g_object_new (GST_TYPE_INDEX_FACTORY, NULL));
-    GST_PLUGIN_FEATURE_NAME (factory) = g_strdup (name);
-  } else {
-    g_object_ref (G_OBJECT (factory));
-  }
-
-  g_free (factory->longdesc);
-  factory->longdesc = g_strdup (longdesc);
-  factory->type = type;
-
-  return factory;
-}
-
-/**
- * gst_index_factory_destroy:
- * @factory: factory to destroy
- *
- * Removes the index from the global list.
- */
-void
-gst_index_factory_destroy (GstIndexFactory * factory)
-{
-  g_return_if_fail (factory != NULL);
-
-  /* we don't free the struct bacause someone might  have a handle to it.. */
-}
-
-/**
- * gst_index_factory_find:
- * @name: name of indexfactory to find
- *
- * Search for an indexfactory of the given name.
- *
- * Returns: #GstIndexFactory if found, NULL otherwise
- */
-GstIndexFactory *
-gst_index_factory_find (const gchar * name)
-{
-  GstPluginFeature *feature;
-
-  g_return_val_if_fail (name != NULL, NULL);
-
-  GST_DEBUG ("gstindex: find \"%s\"", name);
-
-  feature = gst_registry_pool_find_feature (name, GST_TYPE_INDEX_FACTORY);
-  if (feature)
-    return GST_INDEX_FACTORY (feature);
-
-  return NULL;
-}
-
-/**
- * gst_index_factory_create:
- * @factory: the factory used to create the instance
- *
- * Create a new #GstIndex instance from the 
- * given indexfactory.
- *
- * Returns: A new #GstIndex instance.
- */
-GstIndex *
-gst_index_factory_create (GstIndexFactory * factory)
-{
-  GstIndex *new = NULL;
-
-  g_return_val_if_fail (factory != NULL, NULL);
-
-  if (gst_plugin_feature_ensure_loaded (GST_PLUGIN_FEATURE (factory))) {
-    g_return_val_if_fail (factory->type != 0, NULL);
-
-    new = GST_INDEX (g_object_new (factory->type, NULL));
-  }
-
-  return new;
-}
-
-/**
- * gst_index_factory_make:
- * @name: the name of the factory used to create the instance
- *
- * Create a new #GstIndex instance from the 
- * indexfactory with the given name.
- *
- * Returns: A new #GstIndex instance.
- */
-GstIndex *
-gst_index_factory_make (const gchar * name)
-{
-  GstIndexFactory *factory;
-
-  g_return_val_if_fail (name != NULL, NULL);
-
-  factory = gst_index_factory_find (name);
-
-  if (factory == NULL)
-    return NULL;
-
-  return gst_index_factory_create (factory);
-}
