@@ -50,24 +50,6 @@
 
 #include "gstfdsrc.h"
 
-/* the select call is also performed on the control sockets, that way
- * we can send special commands to unblock the select call */
-#define CONTROL_STOP            'S'     /* stop the select call */
-#define CONTROL_SOCKETS(src)   src->control_sock
-#define WRITE_SOCKET(src)      src->control_sock[1]
-#define READ_SOCKET(src)       src->control_sock[0]
-
-#define SEND_COMMAND(src, command)          \
-G_STMT_START {                              \
-  unsigned char c; c = command;             \
-  write (WRITE_SOCKET(src), &c, 1);         \
-} G_STMT_END
-
-#define READ_COMMAND(src, command, res)        \
-G_STMT_START {                                 \
-  res = read(READ_SOCKET(src), &command, 1);   \
-} G_STMT_END
-
 #define DEFAULT_BLOCKSIZE       4096
 
 static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
@@ -78,12 +60,6 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
 GST_DEBUG_CATEGORY_STATIC (gst_fd_src_debug);
 #define GST_CAT_DEFAULT gst_fd_src_debug
 
-static const GstElementDetails gst_fd_src_details =
-GST_ELEMENT_DETAILS ("Disk Source",
-    "Source/File",
-    "Synchronous read from a file",
-    "Erik Walthinsen <omega@cse.ogi.edu>");
-
 enum
 {
   PROP_0,
@@ -107,7 +83,7 @@ _do_init (GType fd_src_type)
   GST_DEBUG_CATEGORY_INIT (gst_fd_src_debug, "fdsrc", 0, "fdsrc element");
 }
 
-GST_BOILERPLATE_FULL (GstFdSrc, gst_fd_src, GstElement, GST_TYPE_PUSH_SRC,
+GST_BOILERPLATE_FULL (GstFdSrc, gst_fd_src, GstPushSrc, GST_TYPE_PUSH_SRC,
     _do_init);
 
 static void gst_fd_src_set_property (GObject * object, guint prop_id,
@@ -119,8 +95,10 @@ static void gst_fd_src_dispose (GObject * obj);
 static gboolean gst_fd_src_start (GstBaseSrc * bsrc);
 static gboolean gst_fd_src_stop (GstBaseSrc * bsrc);
 static gboolean gst_fd_src_unlock (GstBaseSrc * bsrc);
+static gboolean gst_fd_src_unlock_stop (GstBaseSrc * bsrc);
 static gboolean gst_fd_src_is_seekable (GstBaseSrc * bsrc);
 static gboolean gst_fd_src_get_size (GstBaseSrc * src, guint64 * size);
+static gboolean gst_fd_src_do_seek (GstBaseSrc * src, GstSegment * segment);
 
 static GstFlowReturn gst_fd_src_create (GstPushSrc * psrc, GstBuffer ** outbuf);
 
@@ -129,9 +107,12 @@ gst_fd_src_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
+  gst_element_class_set_details_simple (gstelement_class,
+      "Filedescriptor Source",
+      "Source/File",
+      "Read from a file descriptor", "Erik Walthinsen <omega@cse.ogi.edu>");
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&srctemplate));
-  gst_element_class_set_details (gstelement_class, &gst_fd_src_details);
 }
 
 static void
@@ -160,8 +141,10 @@ gst_fd_src_class_init (GstFdSrcClass * klass)
   gstbasesrc_class->start = GST_DEBUG_FUNCPTR (gst_fd_src_start);
   gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_fd_src_stop);
   gstbasesrc_class->unlock = GST_DEBUG_FUNCPTR (gst_fd_src_unlock);
+  gstbasesrc_class->unlock_stop = GST_DEBUG_FUNCPTR (gst_fd_src_unlock_stop);
   gstbasesrc_class->is_seekable = GST_DEBUG_FUNCPTR (gst_fd_src_is_seekable);
   gstbasesrc_class->get_size = GST_DEBUG_FUNCPTR (gst_fd_src_get_size);
+  gstbasesrc_class->do_seek = GST_DEBUG_FUNCPTR (gst_fd_src_do_seek);
 
   gstpush_src_class->create = GST_DEBUG_FUNCPTR (gst_fd_src_create);
 }
@@ -169,10 +152,10 @@ gst_fd_src_class_init (GstFdSrcClass * klass)
 static void
 gst_fd_src_init (GstFdSrc * fdsrc, GstFdSrcClass * klass)
 {
-  fdsrc->fd = 0;
+  fdsrc->fd = -1;
   fdsrc->new_fd = 0;
   fdsrc->seekable_fd = FALSE;
-  fdsrc->uri = g_strdup_printf ("fd://%d", fdsrc->fd);
+  fdsrc->uri = g_strdup_printf ("fd://0");
   fdsrc->curoffset = 0;
 }
 
@@ -192,6 +175,21 @@ gst_fd_src_update_fd (GstFdSrc * src)
 {
   struct stat stat_results;
 
+  /* we need to always update the fdset since it may not have existed when
+   * gst_fd_src_update_fd() was called earlier */
+  if (src->fdset != NULL) {
+    GstPollFD fd = GST_POLL_FD_INIT;
+
+    if (src->fd >= 0) {
+      fd.fd = src->fd;
+      gst_poll_remove_fd (src->fdset, &fd);
+    }
+
+    fd.fd = src->new_fd;
+    gst_poll_add_fd (src->fdset, &fd);
+    gst_poll_fd_ctl_read (src->fdset, &fd, TRUE);
+  }
+
   if (src->fd != src->new_fd) {
     GST_INFO_OBJECT (src, "Updating to fd %d", src->new_fd);
 
@@ -226,20 +224,13 @@ static gboolean
 gst_fd_src_start (GstBaseSrc * bsrc)
 {
   GstFdSrc *src = GST_FD_SRC (bsrc);
-  gint control_sock[2];
 
   src->curoffset = 0;
 
-  gst_fd_src_update_fd (src);
-
-  if (socketpair (PF_UNIX, SOCK_STREAM, 0, control_sock) < 0)
+  if ((src->fdset = gst_poll_new (TRUE)) == NULL)
     goto socket_pair;
 
-  READ_SOCKET (src) = control_sock[0];
-  WRITE_SOCKET (src) = control_sock[1];
-
-  fcntl (READ_SOCKET (src), F_SETFL, O_NONBLOCK);
-  fcntl (WRITE_SOCKET (src), F_SETFL, O_NONBLOCK);
+  gst_fd_src_update_fd (src);
 
   return TRUE;
 
@@ -257,8 +248,10 @@ gst_fd_src_stop (GstBaseSrc * bsrc)
 {
   GstFdSrc *src = GST_FD_SRC (bsrc);
 
-  close (READ_SOCKET (src));
-  close (WRITE_SOCKET (src));
+  if (src->fdset) {
+    gst_poll_free (src->fdset);
+    src->fdset = NULL;
+  }
 
   return TRUE;
 }
@@ -268,7 +261,23 @@ gst_fd_src_unlock (GstBaseSrc * bsrc)
 {
   GstFdSrc *src = GST_FD_SRC (bsrc);
 
-  SEND_COMMAND (src, CONTROL_STOP);
+  GST_LOG_OBJECT (src, "Flushing");
+  GST_OBJECT_LOCK (src);
+  gst_poll_set_flushing (src->fdset, TRUE);
+  GST_OBJECT_UNLOCK (src);
+
+  return TRUE;
+}
+
+static gboolean
+gst_fd_src_unlock_stop (GstBaseSrc * bsrc)
+{
+  GstFdSrc *src = GST_FD_SRC (bsrc);
+
+  GST_LOG_OBJECT (src, "No longer flushing");
+  GST_OBJECT_LOCK (src);
+  gst_poll_set_flushing (src->fdset, FALSE);
+  GST_OBJECT_UNLOCK (src);
 
   return TRUE;
 }
@@ -325,40 +334,21 @@ gst_fd_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
   guint blocksize;
 
 #ifndef HAVE_WIN32
-  fd_set readfds;
   gint retval;
 #endif
 
   src = GST_FD_SRC (psrc);
 
 #ifndef HAVE_WIN32
-  FD_ZERO (&readfds);
-  FD_SET (src->fd, &readfds);
-  FD_SET (READ_SOCKET (src), &readfds);
-
   do {
-    retval = select (FD_SETSIZE, &readfds, NULL, NULL, NULL);
-  } while ((retval == -1 && errno == EINTR));
+    retval = gst_poll_wait (src->fdset, GST_CLOCK_TIME_NONE);
+  } while (retval == -1 && (errno == EINTR || errno == EAGAIN));        /* retry if interrupted */
 
-  if (retval == -1)
-    goto select_error;
-
-  if (FD_ISSET (READ_SOCKET (src), &readfds)) {
-    /* read all stop commands */
-    while (TRUE) {
-      gchar command;
-      int res;
-
-      GST_LOG_OBJECT (src, "reading command");
-
-      READ_COMMAND (src, command, res);
-      if (res < 0) {
-        GST_LOG_OBJECT (src, "no more commands");
-        /* no more commands */
-        break;
-      }
-    }
-    goto stopped;
+  if (retval == -1) {
+    if (errno == EBUSY)
+      goto stopped;
+    else
+      goto select_error;
   }
 #endif
 
@@ -421,7 +411,7 @@ read_error:
   }
 }
 
-gboolean
+static gboolean
 gst_fd_src_is_seekable (GstBaseSrc * bsrc)
 {
   GstFdSrc *src = GST_FD_SRC (bsrc);
@@ -429,7 +419,7 @@ gst_fd_src_is_seekable (GstBaseSrc * bsrc)
   return src->seekable_fd;
 }
 
-gboolean
+static gboolean
 gst_fd_src_get_size (GstBaseSrc * bsrc, guint64 * size)
 {
   GstFdSrc *src = GST_FD_SRC (bsrc);
@@ -453,7 +443,33 @@ could_not_stat:
   {
     return FALSE;
   }
+}
 
+static gboolean
+gst_fd_src_do_seek (GstBaseSrc * bsrc, GstSegment * segment)
+{
+  gint res;
+  gint64 offset;
+  GstFdSrc *src = GST_FD_SRC (bsrc);
+
+  offset = segment->start;
+
+  /* No need to seek to the current position */
+  if (offset == src->curoffset)
+    return TRUE;
+
+  res = lseek (src->fd, offset, SEEK_SET);
+  if (G_UNLIKELY (res < 0 || res != offset))
+    goto seek_failed;
+
+  segment->last_stop = segment->start;
+  segment->time = segment->start;
+
+  return TRUE;
+
+seek_failed:
+  GST_DEBUG_OBJECT (src, "lseek returned %" G_GINT64_FORMAT, offset);
+  return FALSE;
 }
 
 /*** GSTURIHANDLER INTERFACE *************************************************/
@@ -492,7 +508,7 @@ gst_fd_src_uri_set_uri (GstURIHandler * handler, const gchar * uri)
   }
   g_free (protocol);
 
-  if (sscanf (uri, "fd://%d", &fd) != 1)
+  if (sscanf (uri, "fd://%d", &fd) != 1 || fd < 0)
     return FALSE;
 
   src->new_fd = fd;
