@@ -33,12 +33,12 @@
 #include <locale.h>
 #include <glib/gprintf.h>
 
-static char *_name;
+static char *_name = NULL;
 
 static int print_element_info (GstElementFactory * factory,
     gboolean print_names);
 
-void
+static void
 n_print (const char *format, ...)
 {
   va_list args;
@@ -246,16 +246,21 @@ print_interfaces (GType type)
 
   if (ifaces) {
     if (n_ifaces) {
-      g_print ("%s", _name);
+      if (_name)
+        g_print (_name);
       g_print (_("Implemented Interfaces:\n"));
       iface = ifaces;
       while (*iface) {
-        g_print ("%s  %s\n", _name, g_type_name (*iface));
+        if (_name)
+          g_print (_name);
+        g_print ("  %s\n", g_type_name (*iface));
         iface++;
       }
-      g_print ("%s\n", _name);
-      g_free (ifaces);
+      if (_name)
+        g_print (_name);
+      g_print ("\n");
     }
+    g_free (ifaces);
   }
 }
 
@@ -288,19 +293,25 @@ print_element_properties_info (GstElement * element)
     if (param->flags & G_PARAM_READABLE) {
       g_object_get_property (G_OBJECT (element), param->name, &value);
       readable = TRUE;
-      g_print ((first_flag ? "" : ", "));
+      if (!first_flag)
+        g_print (", ");
+      else
+        first_flag = FALSE;
       g_print (_("readable"));
-      first_flag = FALSE;
     }
     if (param->flags & G_PARAM_WRITABLE) {
-      g_print ((first_flag ? "" : ", "));
+      if (!first_flag)
+        g_print (", ");
+      else
+        first_flag = FALSE;
       g_print (_("writable"));
-      first_flag = FALSE;
     }
     if (param->flags & GST_PARAM_CONTROLLABLE) {
-      g_print ((first_flag ? "" : ", "));
+      if (!first_flag)
+        g_print (", ");
+      else
+        first_flag = FALSE;
       g_print (_("controllable"));
-      first_flag = FALSE;
     }
     n_print ("\n");
 
@@ -437,65 +448,88 @@ print_element_properties_info (GstElement * element)
             print_caps (caps, "                           ");
           }
         } else if (G_IS_PARAM_SPEC_ENUM (param)) {
+          GParamSpecEnum *penum = G_PARAM_SPEC_ENUM (param);
           GEnumValue *values;
           guint j = 0;
           gint enum_value;
+          const gchar *def_val_nick = "", *cur_val_nick = "";
 
           values = G_ENUM_CLASS (g_type_class_ref (param->value_type))->values;
           enum_value = g_value_get_enum (&value);
 
           while (values[j].value_name) {
             if (values[j].value == enum_value)
-              break;
+              cur_val_nick = values[j].value_nick;
+            if (values[j].value == penum->default_value)
+              def_val_nick = values[j].value_nick;
             j++;
           }
 
-          n_print ("%-23.23s Enum \"%s\" Current: %d, \"%s\"", "",
-              g_type_name (G_VALUE_TYPE (&value)),
-              enum_value, values[j].value_nick);
+          n_print
+              ("%-23.23s Enum \"%s\" Default: %d, \"%s\" Current: %d, \"%s\"",
+              "", g_type_name (G_VALUE_TYPE (&value)), penum->default_value,
+              def_val_nick, enum_value, cur_val_nick);
 
           j = 0;
           while (values[j].value_name) {
-            g_print ("\n%s%-23.23s    (%d): %-16s - %s", "",
-                _name, values[j].value, values[j].value_nick,
-                values[j].value_name);
+            g_print ("\n");
+            if (_name)
+              g_print (_name);
+            g_print ("%-23.23s    (%d): %-16s - %s", "",
+                values[j].value, values[j].value_nick, values[j].value_name);
             j++;
           }
           /* g_type_class_unref (ec); */
         } else if (G_IS_PARAM_SPEC_FLAGS (param)) {
+          GParamSpecFlags *pflags = G_PARAM_SPEC_FLAGS (param);
           GFlagsValue *values;
           guint j = 0;
           gint flags_value;
-          GString *flags = NULL;
+          GString *cur_flags = NULL, *def_flags = NULL;
 
           values = G_FLAGS_CLASS (g_type_class_ref (param->value_type))->values;
           flags_value = g_value_get_flags (&value);
 
           while (values[j].value_name) {
             if (values[j].value & flags_value) {
-              if (flags) {
-                g_string_append_printf (flags, " | %s", values[j].value_nick);
+              if (cur_flags) {
+                g_string_append_printf (cur_flags, " | %s",
+                    values[j].value_nick);
               } else {
-                flags = g_string_new (values[j].value_nick);
+                cur_flags = g_string_new (values[j].value_nick);
+              }
+            }
+            if (values[j].value & pflags->default_value) {
+              if (def_flags) {
+                g_string_append_printf (def_flags, " | %s",
+                    values[j].value_nick);
+              } else {
+                def_flags = g_string_new (values[j].value_nick);
               }
             }
             j++;
           }
 
-          n_print ("%-23.23s Flags \"%s\" Current: %d, \"%s\"", "",
-              g_type_name (G_VALUE_TYPE (&value)),
-              flags_value, (flags ? flags->str : "(none)"));
+          n_print
+              ("%-23.23s Flags \"%s\" Default: 0x%08x, \"%s\" Current: 0x%08x, \"%s\"",
+              "", g_type_name (G_VALUE_TYPE (&value)), pflags->default_value,
+              (def_flags ? def_flags->str : "(none)"), flags_value,
+              (cur_flags ? cur_flags->str : "(none)"));
 
           j = 0;
           while (values[j].value_name) {
-            g_print ("\n%s%-23.23s    (0x%08x): %-16s - %s", "",
-                _name, values[j].value, values[j].value_nick,
-                values[j].value_name);
+            g_print ("\n");
+            if (_name)
+              g_print (_name);
+            g_print ("%-23.23s    (0x%08x): %-16s - %s", "",
+                values[j].value, values[j].value_nick, values[j].value_name);
             j++;
           }
 
-          if (flags)
-            g_string_free (flags, TRUE);
+          if (cur_flags)
+            g_string_free (cur_flags, TRUE);
+          if (def_flags)
+            g_string_free (def_flags, TRUE);
         } else if (G_IS_PARAM_SPEC_OBJECT (param)) {
           n_print ("%-23.23s Object of type \"%s\"", "",
               g_type_name (param->value_type));
@@ -511,6 +545,20 @@ print_element_properties_info (GstElement * element)
           }
         } else if (param->value_type == G_TYPE_VALUE_ARRAY) {
           n_print ("%-23.23s Array of GValues", "");
+        } else if (GST_IS_PARAM_SPEC_FRACTION (param)) {
+          GstParamSpecFraction *pfraction = GST_PARAM_SPEC_FRACTION (param);
+
+          n_print ("%-23.23s Fraction. ", "");
+
+          g_print ("Range: %d/%d - %d/%d Default: %d/%d ",
+              pfraction->min_num, pfraction->min_den,
+              pfraction->max_num, pfraction->max_den,
+              pfraction->def_num, pfraction->def_den);
+          if (readable)
+            g_print ("Current: %d/%d",
+                gst_value_get_fraction_numerator (&value),
+                gst_value_get_fraction_denominator (&value));
+
         } else {
           n_print ("%-23.23s Unknown type %ld \"%s\"", "", param->value_type,
               g_type_name (param->value_type));
@@ -521,6 +569,8 @@ print_element_properties_info (GstElement * element)
       g_print (" Write only\n");
     else
       g_print ("\n");
+
+    g_value_reset (&value);
   }
   if (num_properties == 0)
     n_print ("  none\n");
@@ -680,17 +730,22 @@ print_pad_info (GstElement * element)
 
   pads = element->pads;
   while (pads) {
+    gchar *name;
+
     pad = GST_PAD (pads->data);
     pads = g_list_next (pads);
 
     n_print ("");
 
+    name = gst_pad_get_name (pad);
     if (gst_pad_get_direction (pad) == GST_PAD_SRC)
-      g_print ("  SRC: '%s'", gst_pad_get_name (pad));
+      g_print ("  SRC: '%s'", name);
     else if (gst_pad_get_direction (pad) == GST_PAD_SINK)
-      g_print ("  SINK: '%s'", gst_pad_get_name (pad));
+      g_print ("  SINK: '%s'", name);
     else
-      g_print ("  UNKNOWN!!!: '%s'", gst_pad_get_name (pad));
+      g_print ("  UNKNOWN!!!: '%s'", name);
+
+    g_free (name);
 
     g_print ("\n");
 
@@ -766,7 +821,11 @@ print_signal_info (GstElement * element)
         if ((k == 0 && !(query->signal_flags & G_SIGNAL_ACTION)) ||
             (k == 1 && (query->signal_flags & G_SIGNAL_ACTION)))
           found_signals = g_slist_append (found_signals, query);
+        else
+          g_free (query);
       }
+      g_free (signals);
+      signals = NULL;
     }
 
     if (found_signals) {
@@ -795,21 +854,25 @@ print_signal_info (GstElement * element)
           g_type_name (query->return_type), g_type_name (type));
 
       for (j = 0; j < query->n_params; j++) {
+        if (_name)
+          g_print (_name);
         if (G_TYPE_IS_FUNDAMENTAL (query->param_types[j])) {
-          g_print (",\n%s%s%s arg%d", _name, indent,
+          g_print (",\n%s%s arg%d", indent,
               g_type_name (query->param_types[j]), j);
         } else if (G_TYPE_IS_ENUM (query->param_types[j])) {
-          g_print (",\n%s%s%s arg%d", _name, indent,
+          g_print (",\n%s%s arg%d", indent,
               g_type_name (query->param_types[j]), j);
         } else {
-          g_print (",\n%s%s%s* arg%d", _name, indent,
+          g_print (",\n%s%s* arg%d", indent,
               g_type_name (query->param_types[j]), j);
         }
       }
 
-      if (k == 0)
-        g_print (",\n%s%sgpointer user_data);\n", _name, indent);
-      else
+      if (k == 0) {
+        if (_name)
+          g_print (_name);
+        g_print (",\n%sgpointer user_data);\n", indent);
+      } else
         g_print (");\n");
 
       g_free (indent);
@@ -874,7 +937,8 @@ print_element_list (gboolean print_all)
           print_element_info (factory, TRUE);
         else
           g_print ("%s:  %s: %s\n", plugin->desc.name,
-              GST_PLUGIN_FEATURE_NAME (factory), factory->details.longname);
+              GST_PLUGIN_FEATURE_NAME (factory),
+              gst_element_factory_get_longname (factory));
       }
 #ifndef GST_DISABLE_INDEX
       else if (GST_IS_INDEX_FACTORY (feature)) {
@@ -970,7 +1034,7 @@ print_plugin_features (GstPlugin * plugin)
 
       factory = GST_ELEMENT_FACTORY (feature);
       n_print ("  %s: %s\n", GST_PLUGIN_FEATURE_NAME (factory),
-          factory->details.longname);
+          gst_element_factory_get_longname (factory));
       num_elements++;
     }
 #ifndef GST_DISABLE_INDEX
@@ -989,14 +1053,16 @@ print_plugin_features (GstPlugin * plugin)
       if (factory->extensions) {
         guint i = 0;
 
-        g_print ("%s type: ", plugin->desc.name);
+        g_print ("%s: %s: ", plugin->desc.name,
+            gst_plugin_feature_get_name (feature));
         while (factory->extensions[i]) {
           g_print ("%s%s", i > 0 ? ", " : "", factory->extensions[i]);
           i++;
         }
         g_print ("\n");
       } else
-        g_print ("%s type: N/A\n", plugin->desc.name);
+        g_print ("%s: %s: no extensions\n", plugin->desc.name,
+            gst_plugin_feature_get_name (feature));
 
       num_types++;
     } else {
@@ -1069,7 +1135,7 @@ print_element_info (GstElementFactory * factory, gboolean print_names)
   if (print_names)
     _name = g_strdup_printf ("%s: ", GST_PLUGIN_FEATURE (factory)->name);
   else
-    _name = "";
+    _name = NULL;
 
   print_factory_details_info (factory);
   if (GST_PLUGIN_FEATURE (factory)->plugin_name) {
@@ -1095,9 +1161,9 @@ print_element_info (GstElementFactory * factory, gboolean print_names)
   print_signal_info (element);
   print_children_info (element);
 
+  gst_object_unref (element);
   gst_object_unref (factory);
-  if (_name[0] != '\0')
-    g_free (_name);
+  g_free (_name);
 
   return 0;
 }
